@@ -3,8 +3,8 @@
 #define MAX_GROUPS 20
 #define MAX_NAME_VALUE 20
 
-#include <netinet/in.h>
 #include <sys/types.h>
+#include <netinet/in.h>
 #include <sys/socket.h>
 #include <sys/un.h>
 #include <errno.h>
@@ -12,6 +12,7 @@
 #include <stdlib.h>
 #include <strings.h>
 #include "testlib.h"
+#include "config.h"
 
 typedef enum
 {
@@ -239,8 +240,13 @@ static GtkWidget *menuitem_festival_terse = NULL;
 static GtkWidget *menuitem_trackmouse = NULL;
 static GtkWidget *menuitem_trackfocus = NULL;
 
+#ifdef HAVE_SOCKADDR_UN_SUN_LEN
+static struct sockaddr_un mag_server = { 0, AF_UNIX , "/tmp/magnifier_socket" };
+static struct sockaddr_un client = { 0 , AF_UNIX, "/tmp/mag_client"};
+#else
 static struct sockaddr_un mag_server = { AF_UNIX , "/tmp/magnifier_socket" };
 static struct sockaddr_un client = { AF_UNIX, "/tmp/mag_client"};
+#endif
 
 /* GUI Information for the output window */
 typedef struct
@@ -265,6 +271,11 @@ _send_to_magnifier(gint x, gint y, gint w, gint h)
   g_print ("sending magnifier: %s\n", buff);
 #endif
 
+#ifdef HAVE_SOCKADDR_UN_SUN_LEN
+  mag_server.sun_len = SUN_LEN(&mag_server);
+  client.sun_len = SUN_LEN(&client);
+#endif
+  
   if((desc=socket(AF_UNIX,SOCK_STREAM,0)) == -1){
     perror("socket");
     return;
@@ -315,6 +326,8 @@ static void _festival_say (const gchar *text)
   static int fd = 0;
   gchar *quoted;
   gchar *p;
+  gchar prefix [100];
+  const gchar *stretch;
 
   fprintf (stderr, "saying %s\n", text);
 
@@ -323,10 +336,14 @@ static void _festival_say (const gchar *text)
       fd = _festival_init ();
     }
 
-  quoted = g_malloc(64+strlen(text)*2);
+  quoted = g_malloc(100+strlen(text)*2);
 
-  strcpy(quoted, "(audio_mode'shutup)\n (SayText \"");
-  p = quoted+31;
+  stretch = g_strdup (g_getenv ("FESTIVAL_STRETCH"));
+  if (!stretch) stretch = "0.75";
+  sprintf (prefix, "(audio_mode'shutup)\n (Parameter.set 'Duration_Stretch %s)\n (SayText \"", stretch);
+  
+  strcpy(quoted, prefix);
+  p = quoted + strlen (prefix);
   while (*text) {
     if ( *text == '\\' || *text == '"' )
       *p = '\\';
@@ -369,10 +386,8 @@ static void _send_to_festival (const gchar *role_name,
   while (name[j])
     {
       ch = name[j++];
-      if (ch != '_')
-        {
-          string[i++] = ch;
-        }
+      if (ch == '_') ch = ' ';
+      string[i++] = ch;
     };
   if ((say_accel) && (strlen (accel) > 0))
     {
@@ -492,7 +507,7 @@ static void _print_accessible (AtkObject *aobject)
   if (use_magnifier)
     {
       gint x, y;
-      gint w=-1, h;
+      gint w=0, h=0;
       
       if (ATK_IS_TEXT (aobject))
         {
@@ -503,23 +518,21 @@ static void _print_accessible (AtkObject *aobject)
 	  atk_text_get_character_extents (ATK_TEXT (aobject), 0,
 					  &x0, &y0, &w0, &h0,
 					  ATK_XY_SCREEN);
-          atk_text_get_character_extents (ATK_TEXT (aobject), len-1,
-					  &xN, &yN, &wN, &hN,
-					  ATK_XY_SCREEN);
-	  x = MIN (x0, xN);
-	  y = MIN (y0, yN);
-	  /*
-	   * due to text coord bugs in gail for GailTextCell
-	   *  we kludge this
-	   */
-	  if (ATK_IS_COMPONENT (aobject))
+          if (len > 0)
 	    {
-	      atk_component_get_extents (ATK_COMPONENT (aobject),
-					 &x, &y, &w, &h,
-					 ATK_XY_SCREEN);
+	      atk_text_get_character_extents (ATK_TEXT (aobject), len-1,
+					      &xN, &yN, &wN, &hN,
+					      ATK_XY_SCREEN);
+	      x = MIN (x0, xN);
+	      y = MIN (y0, yN);
+	      w = MAX (x0+w0, xN+wN) - x;
+	      h = MAX (y0+h0, yN+hN) - y;
+	    }
+          else
+	    {
+	      x = x0;
+	      y = y0;
 	    }
-	  w = MAX (x0+w0, xN+wN) - x;
-	  h = MAX (y0+h0, yN+hN) - y;
         } 
       else if (ATK_IS_COMPONENT (aobject))
         {
@@ -560,6 +573,35 @@ _object_is_ours (AtkObject *aobject)
   return FALSE;
 }
 
+static gchar *
+ferret_get_name_from_container (AtkObject *aobject)
+{
+  gchar *s = NULL;
+  gint n = atk_object_get_n_accessible_children (aobject);
+  gint i = 0;
+  
+  while (!s && (i < n))
+    {
+      AtkObject *child;	    
+      child = atk_object_ref_accessible_child (aobject, i);
+      if (ATK_IS_TEXT (child))
+        {
+		gint count = atk_text_get_character_count (ATK_TEXT (child));
+		s = atk_text_get_text (ATK_TEXT (child),
+				       0,
+				       count);
+        }
+      g_object_unref (child);
+      ++i;	    
+    }
+  
+  if (!s)
+    {
+      s = g_strdup ("");
+    }
+  return s;	
+}
+
 static gint
 _print_object (AtkObject *aobject)
 {
@@ -656,7 +698,22 @@ _print_object (AtkObject *aobject)
             else 
 	      { 
 		text = "";
-		if (!name) name = "no name";
+		if (!name)
+		  {
+		    if (atk_object_get_role (aobject) == ATK_ROLE_TABLE_CELL)
+		      {
+			gchar *cname = ferret_get_name_from_container (aobject);
+			if (cname) name = g_strdup (cname);
+		      }
+		    else if (atk_object_get_role (aobject) == ATK_ROLE_CHECK_BOX)
+		      {
+			name = g_strdup ("check box");
+		      }
+		    else
+		      {
+		        name = "no name";
+		      }
+		  }
 		_send_to_festival (role_name, name, accel_name);
 	      }
           }
@@ -1160,6 +1217,9 @@ _print_text (AtkText *aobject)
       VALUE_STRING);
     g_free(output_str);
 
+    if (caret_offset < 0)
+      return(group_num);
+
     text_val = atk_text_get_text_at_offset (aobject, caret_offset,
                                             ATK_TEXT_BOUNDARY_CHAR,
                                             &start_offset, &end_offset);
